/*************************************************************************
   ising.h: Header file for the Ising model simulator.
									  
   Copyright (c) : Alejandro Luque,1,34+954663083

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
************************************************************************/

#include "stdlib.h"
#include "stdio.h"
#include "stdint.h"
#include "limits.h"

#define N_NEIGHTS 4      /* Numer of next neightbours */

#define DOWN 0           /* Posible states of any spin */
#define UP 1

/* Some probabilities normalized to RAND_MAX
 *
 * Probabilidades para la inicializacion de la red 
 */ 
#define MAX_PROB UINT_MAX
#define ZERO_PROB 0
#define HALF_PROB (MAX_PROB >> 1)

#define TRUE 1
#define FALSE 0

/* Default values for the program settings.
 *
 * Valores por defecto de algunos parametros del programa.
 */
#define DEF_n_min 40
#define DEF_N 1000000
#define DEF_n_sta 20
#define DEF_epsilon2 0.0025

#define m_abs(_x) ((_x) >= 0 ? (_x) : (-(_x)))

#ifdef DEBUG
#define debug(_stmt) _stmt
#else
#define debug(_stmt)
#endif /*DEBUG*/

/* The spin status will be recorded in variables of type spin_t. 
 *
 * El tipo en el que almacenaremos la informacion sobre el espin. 
 */
typedef char spin_t;

/* st_spin containd all the information we need know about any spin
 * in the lattice.
 * 
 * En st_spin va toda la informacion que guardamos sobre un spin.
 */
struct st_spin {
  /* This spin's status.
   * 
   * Estado del espin (arriba o abajo) */
  spin_t status;
  
  /* How many neightbours of this spin ar pointing upward?
   *
   * Numero de vecinos con espin arriba */
  int neight_up;

  /* This array contains pointers to the next neightbours of this spin.
   *
   * Matriz con punteros a los vecinos de este espin 
   */
  struct st_spin *neights[N_NEIGHTS];
};

typedef struct st_spin st_spin;

/* Useful when implementing the output formatting.
 *
 * Util para el formato de salida.
 */
struct st_segment {
  int kind;
  char *text;
  int len;
  struct st_segment *next;
};

typedef struct st_segment st_segment;

#define KIND_PLAIN 0
#define KIND_STOP 1

/* All functions an subroutines are declared here.
 *
 * Declaraciones de todas las funciones. 
 */
int main (int, char**);
void init_global (void);
void do_it (void);
int iterate_N (float*);
st_spin *lat_alloc (int, int);
void init_neights_4 ();
void init_neight_up ();
int init_status (unsigned int);
int count_n_up (void);
void init_p_table (void);
int mc_sim (void);
float delta_ham (spin_t, int);
void toggle_spin (st_spin*);
void init_sta (int);
void reset_sta (void);
int add_sta (float, float*, float*);
st_segment *make_segment (int, char*, int);
st_segment *parse_format (char*);
int seg_fprintf (FILE*, st_segment*);
int save_lattice_oct_ascii (void);
int save_lattice_mat_bin (void);
int put_bin_header (void);
int load_lattice_mat_bin (void);
int read_bin_header (uint32_t*, uint32_t*, uint32_t*, uint32_t*, uint32_t*,
		     char**);
int xfwrite (void*, size_t, size_t, FILE*);
int xfread (void*, size_t, size_t, FILE*);
spin_t read_spin (int);
void fatal (char*);
void warning (char*);
void *xmalloc (size_t);
unsigned int _lineal_rand (unsigned int, unsigned int);
unsigned int mitch_init (unsigned int);
unsigned int mitch_m_init (unsigned int, unsigned int);
unsigned int mitch_rand (void);
unsigned int mitch_m_rand (void);

    
